@@ -6,22 +6,27 @@ from .ipblock import IPBlock
 from .products import Product, ProductType
 from .site import Site
 from .vlan import VLAN
+from .physaddr import PhysAddr
+from .fwtable import FWTable, FWTableEntry
 
-_initialzied = False
+_initialized = False
 
 def initialize():
-    # TODO can these just be at module-level insted of having this be a runtime function?
-    global _initialzied
-    if not _initialzied:
+    # TODO can these just be at module-level instead of having this be a runtime function?
+    global _initialized
+    if not _initialized:
         Asset()
         Device()
+        FWTable()
+        FWTableEntry()
         Interface()
         IPBlock()
         Product()
         ProductType()
         Site()
         VLAN()
-        _initialzied = True
+        PhysAddr()
+        _initialized = True
 
 
 Subnet = IPBlock
@@ -30,5 +35,5 @@ IPContainer = IPBlock
 
 __all__ = [
     'initialize', 'Asset', 'Device', 'Interface', 'IPBlock', 'Subnet', 'IPAddr', 'IPContainer',
-    'Product', 'ProductType', 'Site', 'VLAN', 'NetdotAPIDataclass',
+    'Product', 'ProductType', 'Site', 'VLAN', 'NetdotAPIDataclass', 'PhysAddr', 'FWTable', 'FWTableEntry',
 ]
