@@ -1,6 +1,6 @@
-import copy
+import dataclasses
 import logging
-from typing import Callable, Dict, List, Set
+from typing import Callable, ClassVar, Dict, List, Set
 
 from netdot import parse
 
@@ -17,15 +17,27 @@ def convert_None_values_to_empty_str(diict):
     return ret
 
 
-class NetdotAPIDataclass():
-    _logged_unused_fields: Set = None
-    NETDOT_TABLE_NAME: str
-    NETDOT_MENU_URL_PATH: str  # Useful for generating web_url
-    DEFAULT_DTO_FIELD_PARSERS = {
+@dataclasses.dataclass
+class NetdotAPIDataclass:
+    #
+    # Class variables
+    #
+    _initialized: ClassVar[bool] = False
+    _logged_unused_fields: ClassVar[Set] = None
+    NETDOT_TABLE_NAME: ClassVar[str]
+    NETDOT_MENU_URL_PATH: ClassVar[str]  # Useful for generating web_url
+    DEFAULT_DTO_FIELD_PARSERS: ClassVar = {
         '*_xlink': parse.ID_from_xlink,
     }
+    #
+    # Default dataclass fields
+    #
+    id: int = None
+    _repository: 'netdot.Repository' = None
 
-    _initialized = False
+    # TODO
+    def __eq__(self, other):
+        return type(self) == type(other) and self.id == other.id
 
     def __post_init__(self):
         cls = type(self)
@@ -49,6 +61,14 @@ class NetdotAPIDataclass():
     def _pep8_method_friendly_name(cls):
         return cls.__name__.lower()
 
+    @classmethod
+    def _pep8_method_friendly_name_pluralized(cls):
+        name = cls._pep8_method_friendly_name()
+        if name.endswith('y'):
+            return f'{name[:-1]}ies'
+        else:
+            return f'{name}s'
+
     @classmethod
     def _xlink_field_name(cls):
         # TODO 
@@ -62,11 +82,6 @@ class NetdotAPIDataclass():
         default_parsers_copy.update(additional_parsers)
         return default_parsers_copy
 
-    def __init__(self):
-        # Enable a discoverable API by attaching the 'repository' to every data object it produces.
-        self.id = None
-        self._repository = None
-
     @classmethod
     def foreign_key_field_names(cls) -> List:
         # Get all keys ending with the _xlink suffix. Example: asset_id_xlink 
@@ -99,6 +114,7 @@ class NetdotAPIDataclass():
 
     @classmethod
     def _related_classes(cls) -> Dict[str, 'NetdotAPIDataclass']:
+        # TODO this does not capture many-to-one relationships 
         related_object_types = {}
         for xlink in cls.foreign_key_field_names():
             for other_dataclass in NetdotAPIDataclass.__subclasses__():
@@ -139,7 +155,7 @@ class NetdotAPIDataclass():
             dto_field_parsers (Dict, optional): A dictionary holding field_name -> parse_field() functions. 
                 This is useful for when data returned from the NetDot API is not a string but actually some other data type.
                 Defaults to DEFAULT_STRING_PARSERS. By default, all fields are parsed as strings.
-                field_name may contain a single wildcard at the begining, e.g. *_xlink will expand to used_by_xlink, owner_xlink, parent_xlink, etc... 
+                field_name may contain a single wildcard at the beginning, e.g. *_xlink will expand to used_by_xlink, owner_xlink, parent_xlink, etc... 
 
         Returns:
             NetdotAPIDataclass: The appropriate subclass of NetdotAPIDataclass.
@@ -158,8 +174,9 @@ class NetdotAPIDataclass():
                     # If it is a KeyError indicating 'DTO was missing just this field', just log a DEBUG msg
                     if field_name in e.args and len(e.args) == 1 and not field_name.startswith('_'):
                         logger.debug(
-                            f'DTO recieved from NetDot missing {field_name} for {cls.__module__}.{cls.__name__}')
+                            f'DTO received from NetDot missing {field_name} for {cls.__module__}.{cls.__name__}')
                 except Exception as e:
+                    # TODO why are we ignoring fields instead of ALWAYS warning?
                     if bool(data[field_name]) and data[field_name] != '0':
                         logger.warning(
                             f'Unable to parse {field_name} for {cls.__module__}.{cls.__name__}, value: {data[field_name]} (using {str(string_parser)}) Error: {e}')
@@ -189,9 +206,9 @@ class NetdotAPIDataclass():
     def _log_unused_fields(cls, valid_fields, provided_fields, data_transfer_object):
         unused_fields = provided_fields.difference(valid_fields)
         if unused_fields:
-            # Always log a debug message with ALL the data
             unused_data = {
                 key: data_transfer_object[key] for key in unused_fields}
+            # Always log a debug message with ALL the data
             logger.debug(
                 f'Unparsed data ({cls.__module__}.{cls.__name__}): {unused_data}')
             # If this is the first time we've seen these 'unused field(s)', then log a warning!
@@ -205,11 +222,11 @@ class NetdotAPIDataclass():
         """Expand any 'wildcard' parsers.
         NOTICE: This method updates the provided Dict in place.
 
-        If any of DTO_field_parsers starts with "*", they will be expandeFd to match any of the fields available in DTO_data.
+        If any of DTO_field_parsers starts with "*", they will be expanded to match any of the fields available in DTO_data.
 
         Example:
 
-            Assume we have NatdotAPIDataclass FooAddress.
+            Assume we have NetdotAPIDataclass FooAddress.
 
             >> from dataclasses import dataclass
             >> @dataclass()
@@ -240,23 +257,31 @@ class NetdotAPIDataclass():
 
     @classmethod
     def infer_base_parsers(cls) -> Dict[str, Callable]:
-        return {
+        base_parsers = {
             field_name: dataclass_field.type
             for field_name, dataclass_field in cls.__dataclass_fields__.items()
             if dataclass_field.type is not str
         }
+        for field_name, parser in base_parsers.items():
+            if parser is bool:
+                base_parsers[field_name] = parse.Boolean
+        return base_parsers
 
     @property
     def web_url(self) -> str:
         submenu_web_url = {
             'cable_plant': self.web_url2,
             'management': self.web_url1,
+            'generic': self.web_url2,
         }
         web_url_function = submenu_web_url[self.NETDOT_MENU_URL_PATH]
         return web_url_function()
 
     def web_url1(self) -> str:
-        url_path = f'/{self.NETDOT_MENU_URL_PATH}/{self.NETDOT_TABLE_NAME.lower()}.html'
+        if hasattr(self,'NETDOT_URL_PATH'):
+            url_path = f'/{self.NETDOT_MENU_URL_PATH}/{self.NETDOT_URL_PATH.lower()}.html'
+        else:
+            url_path = f'/{self.NETDOT_MENU_URL_PATH}/{self.NETDOT_TABLE_NAME.lower()}.html'
         return f'{self.server_url}{url_path}?id={self.id}'
 
     def web_url2(self) -> str:
@@ -310,9 +335,9 @@ class NetdotAPIDataclass():
     def _generate_and_attach_others_getAll_methods(cls):
         """Generate all the 'load_XXXs' methods for classes that this class is referenced by.
         """
-        for xlink, xlink_dataclass in cls._related_classes().items():
-            xlink_loader = cls._generate_your_xlink_load_method(xlink, xlink_dataclass)
-            xlink_getter = cls._generate_your_xlink_get_method(xlink, xlink_dataclass)
+        for xlink_dataclass in cls._related_classes().values():
+            xlink_loader = cls._generate_your_xlink_load_method(xlink_dataclass)
+            xlink_getter = cls._generate_your_xlink_get_method(xlink_dataclass)
             xlink_dataclass._attach_methods([xlink_loader, xlink_getter])
 
     @staticmethod
@@ -360,17 +385,18 @@ class NetdotAPIDataclass():
         return xlink_getter
 
     @classmethod
-    def _generate_your_xlink_load_method(my_cls, xlink, other_cls):
+    def _generate_your_xlink_load_method(my_cls, other_cls):
         """Generate the `load_xlinks()` method to be attached to other_cls. 
 
         E.g. will generate load_devices() when my_cls is netdot.Device.
         """
-        def xlink_loader(self) -> List['netdot.T']:
+        def xlink_loader(self) -> List[my_cls]:
             # TODO Should this field be generated outside of this method? Otherwise, "site.devices" will literally not exist until 'site.load_devices()' is called. 
             # For now, we accept that AttributeErrors may occur!
-            xlink_object_field = f'{xlink}s'
+
+            xlink_object_field = my_cls._pep8_method_friendly_name_pluralized()
             if not hasattr(self, xlink_object_field):
-                download_function_name = f'get_{my_cls._pep8_method_friendly_name()}s_by_{other_cls._pep8_method_friendly_name()}'
+                download_function_name = f'get_{my_cls._pep8_method_friendly_name_pluralized()}_by_{other_cls._pep8_method_friendly_name()}'
                 download_function = getattr(self.repository, download_function_name)
                 downloaded_data = download_function(self)
                 setattr(self, xlink_object_field, downloaded_data)                
@@ -381,22 +407,22 @@ class NetdotAPIDataclass():
         Returns:
             List[netdot.{other_cls.__name__}]: The full {other_cls.__name__} object that is related to this {my_cls.__name__}.
         """
-        xlink_loader.__name__ = f'load_{my_cls._pep8_method_friendly_name()}s'
+        xlink_loader.__name__ = f'load_{my_cls._pep8_method_friendly_name_pluralized()}'
         return xlink_loader
 
     @classmethod
-    def _generate_your_xlink_get_method(my_cls, xlink, other_cls):
+    def _generate_your_xlink_get_method(my_cls, other_cls):
         """Generate the `get_xlinks()` method to be attached to dataclass. 
 
         E.g. will generate get_devices() when cls is netdot.Device.
         """
-        xlink_getter = my_cls._generate_your_xlink_load_method(xlink, other_cls)
+        xlink_getter = my_cls._generate_your_xlink_load_method(other_cls)
         xlink_getter.__doc__ = f"""Get the {other_cls.__name__}s associated to this {my_cls.__name__}.
 
         Returns:
             List[netdot.{other_cls.__name__}]: The full {other_cls.__name__} object that is related to this {my_cls.__name__}.
         """
-        xlink_getter.__name__ = f'load_{my_cls._pep8_method_friendly_name()}s'
+        xlink_getter.__name__ = f'load_{my_cls._pep8_method_friendly_name_pluralized()}'
         return xlink_getter
 
     @classmethod
@@ -405,7 +431,7 @@ class NetdotAPIDataclass():
 
         E.g. will generate with_device() when dataclass is netdot.Device.
         """
-        def xlink_setter(self: 'netdot.T', entity: other_cls) -> 'netdot.T':
+        def xlink_setter(self: my_cls, entity: other_cls) -> my_cls:
             setattr(self, my_cls._xlink_object_fieldname(xlink), entity)
             return self
         xlink_setter.__doc__ = f"""Set the {other_cls.__name__}s associated to this {my_cls.__name__}.
