@@ -12,9 +12,10 @@ from netdot.mac_address import MACAddress
 logger = logging.getLogger(__name__)
 
 
-@dataclass()
+@dataclass
 class Device(NetdotAPIDataclass, CSVDataclass):
     NETDOT_MENU_URL_PATH = 'management'
+    NETDOT_URL_PATH = 'device'
     NETDOT_TABLE_NAME = 'Device'
     #
     # Inferred fields
