@@ -2,7 +2,7 @@ import logging
 from dataclasses import dataclass
 from typing import List
 
-import netdot.dataclasses
+from netdot import parse
 from netdot.csv_util import CSVDataclass
 from netdot.dataclasses.base import NetdotAPIDataclass
 from netdot.mac_address import MACAddress
@@ -20,46 +20,54 @@ class Interface(NetdotAPIDataclass, CSVDataclass):
     NETDOT_TABLE_NAME = 'Interface'
     id: int = None
     physaddr: MACAddress = None 
-    name: str = None
-    device: str = None
-    device_xlink: str = None
+    physaddr_xlink: int = None 
     oper_status: str = None
     admin_status: str = None
-    _vlans: 'netdot.dataclasses.VLAN' = None
-    # admin_duplex: str = None
-    # bpdu_filter_enabled: bool = None
-    # bpdu_guard_enabled: bool = None
-    # contactlist: int = None
-    # description: str = None
-    # doc_status: str = None
-    # down_from: datetime = None
-    # down_until: datetime = None
-    # dp_remote_id: str = None
-    # dp_remote_ip: str = None
-    # dp_remote_port: str = None
-    # dp_remote_type: str = None
-    # info: str = None
-    # jack: int = None
-    # jack_char: str = None
-    # loop_guard_enabled: bool = None
-    # monitored: bool = None
-    # monitorstatus: int = None
     neighbor: str = None
-    # neighbor_fixed: bool = None
-    # neighbor_missed: int = None
-    # number: str = None
-    # oper_duplex: str = None
-    # overwrite_descr: bool = None
-    # room_char: str = None
-    # root_guard_enabled: bool = None
-    # snmp_managed: bool = None
-    # speed: int = None
-    # stp_id: str = None
-    # type : str = None
-    # ignore_ip: bool = None
-    # auto_dns: bool = None
-    # circuit: int = None
-    # dlci: str = None
+    admin_duplex: str = None
+    admin_status: str = None
+    bpdu_filter_enabled: bool = None
+    bpdu_guard_enabled: bool = None
+    contactlist: str = None
+    description: str = None
+    device: str = None
+    device_xlink: str = None
+    doc_status: str = None
+    down_from: parse.DateTime = None
+    down_until: parse.DateTime = None
+    dp_remote_id: str = None
+    dp_remote_ip: str = None
+    dp_remote_port: str = None
+    dp_remote_type: str = None
+    info: str = None
+    jack: str = None
+    jack_xlink: str = None
+    jack_char: str = None
+    loop_guard_enabled: bool = None
+    monitored: bool = None
+    monitorstatus: str = None
+    monitorstatus_xlink: int = None
+    name: str = None
+    neighbor: str = None
+    neighbor_xlink: int = None  # TODO, how to wire the neighbor up to look up another interface?
+    neighbor_fixed: bool = None
+    neighbor_missed: int = None
+    number: str = None
+    oper_duplex: str = None
+    oper_status: str = None
+    overwrite_descr: bool = None
+    room: str = None
+    room_xlink: int = None
+    room_char: str = None
+    root_guard_enabled: bool = None
+    snmp_managed: bool = None
+    speed: int = None
+    stp_id: str = None
+    type: str = None
+    ignore_ip: bool = None
+    auto_dns: bool = None
+    circuit: int = None
+    dlci: str = None
 
     @property
     def oper_up(self):
@@ -94,13 +102,3 @@ class Interface(NetdotAPIDataclass, CSVDataclass):
         if len(vlans) != 1:
             return False
         return True
-
-    def get_vlans_count(self) -> List['netdot.VLAN']:
-        if self._vlans is None: 
-            self._vlans = self.repository.get_vlans_count_by_interface(self.id)
-        return self._vlans
-
-    def get_vlans(self) -> List['netdot.VLAN']:
-        if self._vlans is None: 
-            self._vlans = self.repository.get_vlans_by_interface(self.id)
-        return self._vlans
