@@ -11,24 +11,24 @@ def test_all_lowercase():
     assert str(mac) == 'aabbccddeeff'
 
 
-def test_output_with_delimeter():
+def test_output_with_delimiter():
     # Act
     mac = MACAddress('AABBCCDDEEFF')
     # Assert
-    assert mac.format(delimeter=':') == 'aa:bb:cc:dd:ee:ff'
+    assert mac.format(delimiter=':') == 'aa:bb:cc:dd:ee:ff'
 
 
 @pytest.mark.parametrize(
-    'valid_delimeter', [
+    'valid_delimiter', [
         '.',
         ':',
         '-',
         ' ',
     ]
 )
-def test_parse_valid_delimeters(valid_delimeter):
+def test_parse_valid_delimiters(valid_delimiter):
     # Act
-    d = valid_delimeter
+    d = valid_delimiter
     mac = MACAddress(f'AA{d}BB{d}CC{d}DD{d}EE{d}FF')
     # Assert
     assert str(mac) == 'aabbccddeeff'
@@ -44,14 +44,14 @@ def test_whitespace():
     assert str(mac) == 'aabbccddeeff'
 
 
-def test_invalid_delimeter():
+def test_invalid_delimiter():
     # Arrange & Assert
     with pytest.raises(ValueError):
         # Act
         MACAddress('AA+BB+CC+DD+EE+FF')
 
 
-def test_inconsistent_delimeters():
+def test_inconsistent_delimiters():
     # Act
     mac = MACAddress('AA BB CC.DD-EE:FF')
     # Assert
